// ============================================================================
// Copyright (c) 2013 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Dec 23 16:51:25 2013
// ============================================================================

module NIOS_BASIC_DEMO(

      ///////// BUTTON /////////
      input       [1:0]  BUTTON,

      ///////// CLK /////////
      output             CLK_CE,
      output      [2:0]  CLK_OD,
      output      [1:0]  CLK_OS,
      output      [1:0]  CLK_PR,
      output             CLK_RST_n,

      ///////// CLOCK /////////
      output             CLOCK_SCL,
      inout              CLOCK_SDA,

      ///////// CPU /////////
      input              CPU_RESET_n,

      ///////// FAN /////////
      inout              FAN_CTRL,

      ///////// LED /////////
      output      [3:0]  LED,
      output      [3:0]  LED_BRACKET,

      ///////// OSC /////////
      input              OSC_50_B3B,
      input              OSC_50_B3D,
      input              OSC_50_B4A,
      input              OSC_50_B4D,
      input              OSC_50_B7A,
      input              OSC_50_B7D,
      input              OSC_50_B8A,
      input              OSC_50_B8D,

      ///////// SATA /////////
      input              SATA_HOST_RX_p,
      output             SATA_HOST_TX_p,
      input              SATA_REFCLK_p,

      ///////// SFPA /////////
      input              SFPA_LOS,
      input              SFPA_MOD0_PRSNT_n,
      output             SFPA_MOD1_SCL,
      inout              SFPA_MOD2_SDA,
      output      [1:0]  SFPA_RATESEL,
      input              SFPA_RX_p,
      output             SFPA_TXDISABLE,
      input              SFPA_TXFAULT,
      output             SFPA_TX_p,

      ///////// SFP /////////
      input              SFP_REFCLK_p,

      ///////// SW /////////
      input       [1:0]  SW,

      ///////// TEMP /////////
      output             TEMP_CLK,
      inout              TEMP_DATA,
      input              TEMP_INT_n,
      input              TEMP_OVERT_n
);


//=======================================================
//  REG/WIRE declarations
//=======================================================
assign reset_n = 1'b1;

//=======================================================
//  Structural coding
//=======================================================

    TR5_LITE_QSYS u0 (
        .clk_50                                          (OSC_50_B3B),          //clk_50_clk_in.clk
        .reset_n                                         (reset_n),             //clk_50_clk_in_reset.reset_n
		  
		  // temperature
        .temp_scl_external_connection_export             (TEMP_CLK),            //temp_clk_external_connection.export
        .temp_sda_external_connection_export             (TEMP_DATA),           //temp_data_external_connection.export
        .in_port_to_the_temp_int_n                       (TEMP_INT_n),          //temp_int_n_external_connection.export
        .in_port_to_the_temp_overt_n                     (TEMP_OVERT_n),        //temp_overt_n_external_connection.export
		  

		  // generic io
        .in_port_to_the_button                           (BUTTON),              //button_external_connection.export
        .out_port_from_the_led                           (LED),                 //led_external_connection.export
        .sw_external_connection_export                   (SW),                  //sw_external_connection.export
		  
		  //extern clock control
        .cdcm_od_external_connection_export              (CLK_OD),              //cdcm_od_external_connection.export
        .cdcm_pr_external_connection_export              (CLK_PR),              //cdcm_pr_external_connection.export
        .cdmc_reset_n_external_connection_export         (CLK_RST_n),           //cdmc_reset_n_external_connection.export
		  
        .clk_i2c_scl_external_connection_export          (CLOCK_SCL),           //clk_i2c_scl_external_connection.export
        .clk_i2c_sda_external_connection_export          (CLOCK_SDA),           //clk_i2c_sda_external_connection.export
		  
		  // external clock measure
		  //.ref_clock_sata_count_clk_in_ref_export        (OSC_50_B3B),          //ref_clock_sata_count_clk_in_ref.export
        .ref_clock_sata_count_clk_in_ref_export          (OSC_50_B4A),          //ref_clock_sata_count_clk_in_ref.export
        .ref_clock_sata_count_clk_in_target_export       (SATA_REFCLK_p),       //ref_clock_sata_count_clk_in_target.export
        .ref_clock_10g_count_clk_in_ref_export           (OSC_50_B3B),          //ref_clock_10g_count_clk_in_ref.export
        .ref_clock_10g_count_clk_in_target_export        (SFP_REFCLK_p),        //ref_clock_10g_count_clk_in_target.export

		  
			
    );
	 
////////////////////////////////////////
// Dummy xcvr to avoid compile error
xcvr_dummy xcvr_dummy_sata(
		.pll_ref_clk(SATA_REFCLK_p),            //     pll_ref_clk.clk
		.tx_serial_data(SATA_HOST_TX_p),       //     tx_serial_data.export
		.rx_serial_data(SATA_HOST_RX_p),       //     rx_serial_data.export
		
	);
	

xcvr_dummy xcvr_dummy_10g(
		.pll_ref_clk(SFP_REFCLK_p),            //      pll_ref_clk.clk
		.tx_serial_data(SFPA_TX_p),            //      tx_serial_data.export
		.rx_serial_data(SFPA_RX_p),            //      rx_serial_data.export
		
	);	 
	 
///////////////////////////////////////////////	 
//CD CM
assign CLK_CE = 1'b1;
assign CLK_OS = 2'b01; // LVDS, OSC_OUT off	 


	
////////////////////////////////////////
// FAN: turn on
assign FAN_CTRL = 1'b1; 	




endmodule
