# (C) 2001-2012 Altera Corporation. All rights reserved.
# Your use of Altera Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Altera Program License Subscription 
# Agreement, Altera MegaCore Function License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Altera and sold by 
# Altera or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


#############################################################################################
#
# This is an example file for compilation/simulation of Low Latency PHY instance in Modelsim 
#
# You can modify this script and use it to compile/simulate your design depending on  
# the following Modelsim license options:
# (1) Mixed language license 
#    - Top-level PHY IP variant can be in Verilog or VHDL
#    - Underlying PHY IP files are in plaintext Verilog 
#    - Testbench can be in Verilog or VHDL
#      
# (2) Non-mixed language license   
#     (a) Verilog Only:
#        - Top-level PHY IP variant should be in Verilog
#        - Underlying PHY IP files are in plaintext Verilog
#        - Testbench should be in Verilog
#
#     (b) VHDL Only:
#        - Top-level PHY IP variant should be in VHDL
#        - Underlying PHY IP files are in encrypted Verilog
#        - Testbench should be in VHDL
#
# (3) Quartus encrypted some files for copyright. Those file will be encrypted for different simulator
# 
# Please do the following changes in this script in order to use it:
#  - Set the language (verilog or vhdl) of the generated PHY IP variant  
#  - Set your top-level PHY IP variant name
#  - Set your top-level testbench name
#  - Compile your other design files and your testbench. You would use vlog or vcom according 
#    to the language of your design files/testbench and your Modelsim license. 
# 
# Note that the PHY IP files are correctly compiled in this script according to the the 
# "language" variable you set.
#
##############################################################################################
onerror {exit}

set QUARTUS_ROOTDIR $env(QUARTUS_ROOTDIR)

#################################################################################
##
## Set your language, simulator and top level design name here
## e.g. vsim -c -do "do ./test_sim/altera_xcvr_low_latency_phy/modelsim_example_script.tcl verilog stratixv modelsim test test_tst"
## 
#################################################################################
# language = verilog (verilog variant of the PHY IP) or vhdl (vhdl variant of the PHY IP)
# defaulted to verilog
set language $1
if {$language != "verilog" && $language != "vhdl"} {
	echo "invalid setting for language: $language. valid settings are verilog and vhdl"
	exit
}

# family = stratixiv or stratixv
# defaulted to stratixv
set family $2
if {$family != "stratixiv" && $family != "stratixv"} {
	echo "invalid setting for family: $family. valid settings are stratixiv and stratixv"
	exit
}

# simulator = modelsim or VCS
# defaulted to modelsim
set simulator $3
if {$simulator != "modelsim" && $simulator != "vcs" && $simulator != "ncsim"} {
	echo "invalid setting for simulator: $simulator. valid settings are modelsim and VCS"
	exit
}

## Set your top level design name here
##set dut_name <top level design name as generted in Quartus or Qsys>
set dut_name $4
if {$dut_name == ""} {
	echo "Missing top level design name: $dut_name"
}

## Set your test bench name here
##set tb_name <test bench>
set tb_name $5
if {$tb_name == ""} {
	echo "Missing test bench name: $tb_name"
}

#################################################################################
## Set directory path according to simulator
## verilog_sim_file_dir for Quartus unecrypted files
## vhdl_sim_file_dir is for Quartus ecrypted files
#################################################################################
## Modelsim Verilog/Modelsim VHDL with Mixed language license 
if { $simulator == "modelsim" } {
    set verilog_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy
    set vhdl_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy/mentor
    set simulator_lib mentor
} elseif { $simulator == "VCS" } {
    set verilog_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy 
    set vhdl_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy
    set simulator_lib synopsys
} elseif { $simulator == "Aldec" } {
    set verilog_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy 
    set vhdl_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy
    set simulator_lib aldec
} elseif { $simulator == "ncsim" } {
    set verilog_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy 
    set vhdl_sim_file_dir ./${dut_name}_sim/altera_xcvr_low_latency_phy
    set simulator_lib cadence
}

exec rm -rf work
vlib work


if {$language == "verilog"} {
    ###########################################
    # Common library files 
    ###########################################
    vlog $QUARTUS_ROOTDIR/eda/sim_lib/220model.v
    vlog $QUARTUS_ROOTDIR/eda/sim_lib/altera_primitives.v
    vlog $QUARTUS_ROOTDIR/eda/sim_lib/sgate.v
    vlog $QUARTUS_ROOTDIR/eda/sim_lib/altera_mf.v
    
    #header file
    vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/alt_xcvr_csr_common_h.sv
    #vlog  +incdir+$vhdl_sim_file_dir  -sv    $vhdl_sim_file_dir/alt_xcvr_csr_pcs8g_h.sv
    
    foreach f [glob $verilog_sim_file_dir/*_h.sv ] {
        if { [ string match "*alt_xcvr_csr_common_h.sv" $f ] == 0} {
            vlog -sv $f
        }
    }
    
    #common function
    vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/altera_xcvr_functions.sv


    if {$family == "stratixv"} {
        ###########################################
        # Stratix V library files 
        ###########################################
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/altera_lnsim.sv
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/stratixv_atoms.v
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/stratixv_hssi_atoms.v
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_atoms_ncrypt.v
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_hssi_atoms_ncrypt.v
        
        foreach f [glob $verilog_sim_file_dir/*.v ] {
            vlog $f
        }
        
        foreach f [glob $verilog_sim_file_dir/*.sv ] {
            if { [ string match "*_h.sv" $f ] == 0} {
                if { [ string match "*altera_xcvr_functions.sv" $f ] == 0} { 
                    vlog -sv $f
                }
            }
        }

    } else {
        ###########################################
        # Stratix IV library files 
        ###########################################
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/stratixiv_hssi_atoms.v
        
        vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/alt_pma_tgx.sv
        vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/alt_pma_functions.sv
        
        vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/alt_xcvr_reconfig_siv.sv 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_analog_tgx.v
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_offset_cancellation_tgx.v
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_eyemon_tgx.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_eyemon.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_dfe_tgx.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_dfe.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_xcvr_reconfig_basic_tgx.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_mutex_acq.v 
        vlog  +incdir+$verilog_sim_file_dir         $verilog_sim_file_dir/alt_dprio.v
    }
  
} else {
    vmap altera work
    vmap altera_mf work
    vmap lpm work
    vmap sgate work
    vmap stratixiv_hssi work
    
    ###########################################
    # Common library files 
    ###########################################
    
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/220pack.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/220model.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/altera_primitives_components.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/altera_primitives.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/sgate_pack.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/sgate.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/altera_mf_components.vhd
    vcom $QUARTUS_ROOTDIR/eda/sim_lib/altera_mf.vhd
    
    #header file
    vlog  +incdir+$vhdl_sim_file_dir  -sv    $vhdl_sim_file_dir/alt_xcvr_csr_common_h.sv
    #vlog  +incdir+$vhdl_sim_file_dir  -sv    $vhdl_sim_file_dir/alt_xcvr_csr_pcs8g_h.sv
    
    foreach f [glob $vhdl_sim_file_dir/*_h.sv ] {
        if { [ string match "*alt_xcvr_csr_common_h.sv" $f ] == 0} {
            vlog -sv $f
        }
    }
    
    #common function
    vlog  +incdir+$vhdl_sim_file_dir  -sv    $vhdl_sim_file_dir/altera_xcvr_functions.sv
    
   
  
    if {$family == "stratixv"} {
        ###########################################
        # Stratix V library files 
        ###########################################
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_atoms_ncrypt.v
        vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_hssi_atoms_ncrypt.v
        if { $simulator == "modelsim" } {
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/altera_lnsim_for_vhdl.sv
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_atoms_for_vhdl.v
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/$simulator_lib/stratixv_hssi_atoms_for_vhdl.v
        } else {
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/altera_lnsim.sv
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/stratixv_atoms.v
            vlog $QUARTUS_ROOTDIR/eda/sim_lib/stratixv_hssi_atoms.v
        }
        
        #plaintext file
        #vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/*.v
        #vlog  +incdir+$verilog_sim_file_dir  -sv    $verilog_sim_file_dir/*.sv
        
        #encrypted files
        #vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/*.v
        #vlog  +incdir+$vhdl_sim_file_dir  -sv   $vhdl_sim_file_dir/*.sv
        
        foreach f [glob $vhdl_sim_file_dir/*.v ] {
            vlog $f
        }
        
        foreach f [glob $vhdl_sim_file_dir/*.sv ] {
            if { [ string match "*_h.sv" $f ] == 0} {
                if { [ string match "*altera_xcvr_functions.sv" $f ] == 0} { 
                    vlog -sv $f
                }
            }
        }
        
    } else {
        ###########################################
        # Stratix IV library files 
        ###########################################
        vcom $QUARTUS_ROOTDIR/eda/sim_lib/stratixiv_hssi_components.vhd
        vcom $QUARTUS_ROOTDIR/eda/sim_lib/stratixiv_hssi_atoms.vhd
        
        vlog  +incdir+$vhdl_sim_file_dir  -sv   $vhdl_sim_file_dir/alt_pma_tgx.sv
        vlog  +incdir+$vhdl_sim_file_dir  -sv   $vhdl_sim_file_dir/alt_pma_functions.sv
        
        vlog  +incdir+$vhdl_sim_file_dir  -sv   $vhdl_sim_file_dir/alt_xcvr_reconfig_siv.sv 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_analog_tgx.v
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_offset_cancellation_tgx.v
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_eyemon_tgx.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_eyemon.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_dfe_tgx.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_dfe.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_xcvr_reconfig_basic_tgx.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_mutex_acq.v 
        vlog  +incdir+$vhdl_sim_file_dir        $vhdl_sim_file_dir/alt_dprio.v
    }
}


###########################################
# Add your design file here
###########################################
if {$language == "verilog"} {
    vlog $dut_name.v
} else {
    vcom $dut_name.vhd
}


###########################################
# Add your test bench here
###########################################
vlog $tb_name.v
if {$language == "verilog"} {
    vsim work.$tb_name
} else {
    vsim -novopt work.$tb_name
}

###########################################
# Run and Quit your simulator
###########################################
run -all
quit -sim
exit
