// ============================================================================
// Copyright (c) 2013 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Dec 23 16:51:25 2013
// ============================================================================
// Major Functions:
//  This code is using for configuring SI570 Controller
// ============================================================================
// Design Description:   TR5_LITE_SI570_Controller
// ===========================================================================
// Revision History :
// ============================================================================
// Ver  :| Author     :| Mod. Date :| Changes Made:
// V1.0 :| Johnny Fan :| 12/02/20  :| Initial Version
// V2.0 :| XinXian    :| 13/12/30  :| Update to Q13.1 for TR5_Lite_revB
// V3.0 :| Dee Zeng   :| 14/10/23  :| Update si570_controller
// ============================================================================

//`define ENABLE_DDR3A
//`define ENABLE_DDR3B
//`define ENABLE_QDRII
`define ENABLE_SFP
//`define ENABLE_PCIE
//`define ENABLE_SATA

module TR5_LITE_SI570_Controller(

      ///////// BUTTON /////////
      input       [1:0]  BUTTON,

      ///////// CLK /////////
      output             CLK_CE,
      output      [2:0]  CLK_OD,
      output      [1:0]  CLK_OS,
      output      [1:0]  CLK_PR,
      output             CLK_RST_n,

      ///////// CLOCK /////////
      output             CLOCK_SCL,
      inout              CLOCK_SDA,

      ///////// CPU /////////
      input              CPU_RESET_n,

`ifdef ENABLE_DDR3A
      ///////// DDR3A /////////
      output      [15:0] DDR3A_A,
      output      [2:0]  DDR3A_BA,
      output             DDR3A_CAS_n,
      output             DDR3A_CK,
      output      [1:0]  DDR3A_CKE,
      output             DDR3A_CK_n,
      output      [1:0]  DDR3A_CS_n,
      output      [1:0]  DDR3A_DM,
      inout       [15:0] DDR3A_DQ,
      inout       [1:0]  DDR3A_DQS,
      inout       [1:0]  DDR3A_DQS_n,
      output      [1:0]  DDR3A_ODT,
      output             DDR3A_RAS_n,
      output             DDR3A_RESET_n,
      output             DDR3A_WE_n,
`endif /*ENABLE_DDR3A*/

`ifdef ENABLE_DDR3B
      ///////// DDR3B /////////
      output      [15:0] DDR3B_A,
      output      [2:0]  DDR3B_BA,
      output             DDR3B_CAS_n,
      output             DDR3B_CK,
      output      [1:0]  DDR3B_CKE,
      output             DDR3B_CK_n,
      output      [1:0]  DDR3B_CS_n,
      output      [1:0]  DDR3B_DM,
      inout       [15:0] DDR3B_DQ,
      inout       [1:0]  DDR3B_DQS,
      inout       [1:0]  DDR3B_DQS_n,
      output      [1:0]  DDR3B_ODT,
      output             DDR3B_RAS_n,
      output             DDR3B_RESET_n,
      output             DDR3B_WE_n,
`endif /*ENABLE_DDR3B*/

      ///////// FAN /////////
      inout              FAN_CTRL,

      ///////// FLASH /////////
      output             FLASH_ADV_n,
      output      [1:0]  FLASH_CE_n,
      output             FLASH_CLK,
      output             FLASH_OE_n,
      input       [1:0]  FLASH_RDY_BSY_n,
      output             FLASH_RESET_n,
      output             FLASH_WE_n,

      ///////// FSM /////////
      output      [26:0] FSM_A,
      inout       [31:0] FSM_D,

      ///////// LED /////////
      output      [3:0]  LED,
      output      [3:0]  LED_BRACKET,

      ///////// OSC /////////
      input              OSC_50_B3B,
      input              OSC_50_B3D,
      input              OSC_50_B4A,
      input              OSC_50_B4D,
      input              OSC_50_B7A,
      input              OSC_50_B7D,
      input              OSC_50_B8A,
      input              OSC_50_B8D,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      input              PCIE_PERST_n,
      input              PCIE_REFCLK_p,
      input       [7:0]  PCIE_RX_p,
      input              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output      [7:0]  PCIE_TX_p,
      output             PCIE_WAKE_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QDRII
      ///////// QDRIIA /////////
      output      [20:0] QDRIIA_A,
      output      [1:0]  QDRIIA_BWS_n,
      input              QDRIIA_CQ_n,
      input              QDRIIA_CQ_p,
      output      [17:0] QDRIIA_D,
      output             QDRIIA_DOFF_n,
      output             QDRIIA_K_n,
      output             QDRIIA_K_p,
      output             QDRIIA_ODT,
      input       [17:0] QDRIIA_Q,
      input              QDRIIA_QVLD,
      output             QDRIIA_RPS_n,
      output             QDRIIA_WPS_n,

      ///////// QDRIIB /////////
      output      [20:0] QDRIIB_A,
      output      [1:0]  QDRIIB_BWS_n,
      input              QDRIIB_CQ_n,
      input              QDRIIB_CQ_p,
      output      [17:0] QDRIIB_D,
      output             QDRIIB_DOFF_n,
      output             QDRIIB_K_n,
      output             QDRIIB_K_p,
      output             QDRIIB_ODT,
      input       [17:0] QDRIIB_Q,
      input              QDRIIB_QVLD,
      output             QDRIIB_RPS_n,
      output             QDRIIB_WPS_n,

      ///////// QDRIIC /////////
      output      [20:0] QDRIIC_A,
      output      [1:0]  QDRIIC_BWS_n,
      input              QDRIIC_CQ_n,
      input              QDRIIC_CQ_p,
      output      [17:0] QDRIIC_D,
      output             QDRIIC_DOFF_n,
      output             QDRIIC_K_n,
      output             QDRIIC_K_p,
      output             QDRIIC_ODT,
      input       [17:0] QDRIIC_Q,
      input              QDRIIC_QVLD,
      output             QDRIIC_RPS_n,
      output             QDRIIC_WPS_n,

      ///////// QDRIID /////////
      output      [20:0] QDRIID_A,
      output      [1:0]  QDRIID_BWS_n,
      input              QDRIID_CQ_n,
      input              QDRIID_CQ_p,
      output      [17:0] QDRIID_D,
      output             QDRIID_DOFF_n,
      output             QDRIID_K_n,
      output             QDRIID_K_p,
      output             QDRIID_ODT,
      input       [17:0] QDRIID_Q,
      input              QDRIID_QVLD,
      output             QDRIID_RPS_n,
      output             QDRIID_WPS_n,
`endif //ENABLE_QDRII
		
      ///////// RZQ /////////
      input              RZQ_0,
      input              RZQ_1,
      input              RZQ_4,
      input              RZQ_5,


      ///////// RS422 /////////
      output             RS422_DE,
      input              RS422_DIN,
      output             RS422_DOUT,
      output             RS422_RE_n,
      output             RS422_TE,

`ifdef ENABLE_SATA
      ///////// SATA /////////
      input              SATA_HOST_RX_p,
      output             SATA_HOST_TX_p,
      input              SATA_REFCLK_p,
`endif /*ENABLE_SATA*/

`ifdef ENABLE_SFP
      ///////// SFP1G /////////
     // input              SFP1G_REFCLK_p,

      ///////// SFPA /////////
      input              SFPA_LOS,
      input              SFPA_MOD0_PRSNT_n,
      output             SFPA_MOD1_SCL,
      inout              SFPA_MOD2_SDA,
      output      [1:0]  SFPA_RATESEL,
      input              SFPA_RX_p,
      output             SFPA_TXDISABLE,
      input              SFPA_TXFAULT,
      output             SFPA_TX_p,

      ///////// SFPB /////////
      input              SFPB_LOS,
      input              SFPB_MOD0_PRSNT_n,
      output             SFPB_MOD1_SCL,
      inout              SFPB_MOD2_SDA,
      output      [1:0]  SFPB_RATESEL,
      input              SFPB_RX_p,
      output             SFPB_TXDISABLE,
      input              SFPB_TXFAULT,
      output             SFPB_TX_p,

      ///////// SFP /////////
      input              SFP_REFCLK_p,
`endif /*ENABLE_SFP*/

      ///////// SW /////////
      input       [1:0]  SW,

      ///////// TEMP /////////
      output             TEMP_CLK,
      inout              TEMP_DATA,
      input              TEMP_INT_n,
      input              TEMP_OVERT_n
);


//=======================================================
//  REG/WIRE declarations
//=======================================================
wire user_reset_n;
wire user_si570_reset_n;

// configure external pll Si570 which provides xcvr ref clock
wire [2:0]	clk_type;
wire si570_controller_start;
wire reset_n;
wire clk_125;

wire phy_mgmt_clk_reset;
wire [31:0] tx_pattern;

//=======================================================
//  Structural coding
//=======================================================

assign user_reset_n = BUTTON[0];
assign user_si570_reset_n = BUTTON[1];

////////////////////////////////////////
//assign clk_type = 3'b000;  //100.0 MHZ
//assign clk_type = 3'b001;  //125.0 MHZ
//assign clk_type = 3'b010;  //156.25.0 MHZ
//assign clk_type = 3'b011;  //250 MHZ
//assign clk_type = 3'b100;  //312.5 MHZ
//assign clk_type = 3'b101;  //322.265625 MHZ
assign clk_type = 3'b110;  //644.53125 MHZ


si570_controller u1(

.iCLK(OSC_50_B3B), 										// system   clock 50mhz 
.iRST_n(user_reset_n), 									// system reset
.iStart(si570_controller_start),						// giving 1 50Mhz cycle high pulse to set frequency
.iFREQ_MODE(clk_type),									// clock frequency mode   000:100Mhz, 001: 125Mhz, 010:156.25Mhz, 011:250Mhz, 100:312.5Mhz , 101:322.26Mhz , 110:644.53Mhz ,111:100Mhz 
.I2C_CLK(CLOCK_SCL),
.I2C_DATA(CLOCK_SDA),
.oController_Ready(),
);


edge_detector edge_detector(

.iCLK(OSC_50_B3B),
.iRST_n(user_reset_n),
.iIn(BUTTON[1]),
.oDEBOUNCE_OUT(si570_controller_start),
);


////////////////////////////////////////
// generate clk125 for phy_mgmt

mgmt_pll mgmt_pll_inst(
		.refclk(OSC_50_B3B),  			 				 // refclk.clk
		.rst(1'b0),      						          // reset.reset
		.outclk_0(clk_125), 					          // outclk0.clk
		.locked(reset_n)    								 // locked.export
	);

////////////////////////////////////////
// XCVR


assign phy_mgmt_clk_reset = ~reset_n | ~user_reset_n;
assign tx_pattern = 32'h12345678;


LOW_LATENCY_XCVR_1x32 SFP_A(
		.phy_mgmt_clk(clk_125),                    // phy_mgmt_clk.clk
		.phy_mgmt_clk_reset(phy_mgmt_clk_reset),   // phy_mgmt_clk_reset.reset
		.phy_mgmt_address(),     						 // phy_mgmt.address
		.phy_mgmt_read(),        				       //.read
		.phy_mgmt_readdata(),    					  	 //.readdata
		.phy_mgmt_waitrequest(), 				       //.waitrequest
		.phy_mgmt_write(),       						 //.write
		.phy_mgmt_writedata(),  				       //.writedata
		.tx_ready(),            		 				 // tx_ready.export
		.rx_ready(),             				       // rx_ready.export
		.pll_ref_clk(SFP_REFCLK_p),         		 // pll_ref_clk.clk
		.pll_locked(),           						 // pll_locked.export
		.tx_serial_data(SFPA_TX_p),      			 // tx_serial_data.export
		.rx_serial_data(SFPA_RX_p),      			 // rx_serial_data.export
		.rx_is_lockedtoref(),   						 // rx_is_lockedtoref.export
		.rx_is_lockedtodata(),   						 // rx_is_lockedtodata.export
		.tx_clkout(),           						 // tx_clkout.export
		.rx_clkout(),            						 // rx_clkout.export
		.tx_parallel_data(tx_pattern),             // tx_parallel_data.export
		.rx_parallel_data(),                       // rx_parallel_data.export
		.reconfig_from_xcvr(),                     // reconfig_from_xcvr.reconfig_from_xcvr
		.reconfig_to_xcvr()     					    // reconfig_to_xcvr.reconfig_to_xcvr
	);
	
/// SFP-B

LOW_LATENCY_XCVR_1x32 SFP_B(
		.phy_mgmt_clk(clk_125),                    // phy_mgmt_clk.clk
		.phy_mgmt_clk_reset(phy_mgmt_clk_reset),   // phy_mgmt_clk_reset.reset
		.phy_mgmt_address(),                       // phy_mgmt.address
		.phy_mgmt_read(),                          //.read
		.phy_mgmt_readdata(),                      //.readdata
		.phy_mgmt_waitrequest(),                   //.waitrequest
		.phy_mgmt_write(),                         //.write
		.phy_mgmt_writedata(),                     //.writedata
		.tx_ready(),                               // tx_ready.export
		.rx_ready(),            						 // rx_ready.export
		.pll_ref_clk(SFP_REFCLK_p),          		 // pll_ref_clk.clk
		.pll_locked(),          						 // pll_locked.export
		.tx_serial_data(SFPB_TX_p),      			 // tx_serial_data.export
		.rx_serial_data(SFPB_RX_p),       			 // rx_serial_data.export
		.rx_is_lockedtoref(),    						 // rx_is_lockedtoref.export
		.rx_is_lockedtodata(),   						 // rx_is_lockedtodata.export
		.tx_clkout(),            						 // tx_clkout.export
		.rx_clkout(),            						 // rx_clkout.export
		.tx_parallel_data(tx_pattern),     			 // tx_parallel_data.export
		.rx_parallel_data(),     						 // rx_parallel_data.export
		.reconfig_from_xcvr(),  						 // reconfig_from_xcvr.reconfig_from_xcvr
		.reconfig_to_xcvr()      						 // reconfig_to_xcvr.reconfig_to_xcvr
	);	

///////////////////////////////////////////
// generate heart beat for clock
wire xcvr_refclk_hb;
heart_beat heart_beat_xcvr_refclk(
	.clk(SFP_REFCLK_p),
	.led(xcvr_refclk_hb)
);

wire clk125_hb;
heart_beat heart_beat_clk_125(
	.clk(clk_125),
	.led(clk125_hb)
);


//////////////////////////////////////////
// led indication
assign LED[0] = phy_mgmt_clk_reset?1'b0:1'b1;	 // on if reset assert
assign LED[2] = clk125_hb; 							 // heart beat of clk125
assign LED[3] = xcvr_refclk_hb; 						 // heart beat of xcvr refclk


//////////////////////////////////////////
// turn on the fan
assign FAN_CTRL = 1'b1;

endmodule
